local CG = LibStub("AceAddon-3.0"):NewAddon("CoolGlow", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0", "AceTimer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("CoolGlow")
local LCG = LibStub('LibCustomGlow-1.0')

CG.version = GetAddOnMetadata("CoolGlow", "Version")

local defaults = {
	char = {
		Style = "Pixel",
		Color = {r = 0.95, g = 0.95, b = 0.32, a = 1},
		Button = {
			Frequency = 0.2,
		},
		Pixel = {
			NumLines = 8,
			Frequency = 0.45,
			Length = 8,
			Thickness = 2,
			XOffset = 0,
			YOffset = 0
		},
		AutoCast = {
			NumParticles = 8,
			Frequency = 0.45,
			Scale = 1,
			XOffset = 0,
			YOffset = 0
		},
	}
}

local options = {
	type = "group",
	handler = CG,
	name = L["Cool Glow"],
	childGroups = "tab",
	args = {
		Style = {
			order = 1,
			type = "select",
			name = L["Style"],
			get = function(info) return CG.db.char.style end,
			set = function(info, value) CG.db.char.style = value end,
			values ={
				["Button"] = L["Button Glow"],
				["Pixel"] = L["Pixel Glow"],
				["AutoCast"] = L["AutoCast Glow"],
			}
		},
		Color = {
			order = 2,
			type = "color",
			name = L["Color"],
			hasAlpha = true,
			get = function(info)
				local t = CG.db.char.Color
				return t.r, t.g, t.b, t.a
			end,
			set = function(info, r, g, b, a)
				local t = CG.db.char.Color
				t.r, t.g, t.b, t.a = r, g, b, a
				CG:ActionbarGlow()
			end,
		},
		Button = {
			order = 1,
			type = "group",
			name = L["Button Glow"],
			get = function(info) return CG.db.char.Button[ info[#info] ] end,
			set = function(info, value) CG.db.char.Button[ info[#info] ] = value; CG:ActionbarGlow() end,
			args = {
				Frequency = {
					order = 1,
					type = "range",
					name = L["Frequency"],
					desc = L["Animation speed. Negative values will rotate anti-clockwise."],
					min = 0, max = 1, step = 0.1,
				},
			},
		},
		Pixel = {
			type = "group",
			name = L["Pixel Glow"],
			order = 2,
			get = function(info) return CG.db.char.Pixel[ info[#info] ] end,
			set = function(info, value) CG.db.char.Pixel[ info[#info] ] = value; CG:ActionbarGlow() end,
			args = {
				NumLines = {
					order = 1,
					type = "range",
					name = L["Num Lines"],
					desc = L["Defines the number of lines the glow will spawn."],
					min = 4, max = 16, step = 1,
				},
				Frequency = {
					order = 2,
					type = "range",
					name = L["Frequency"],
					desc = L["Sets the animation speed of the glow. Negative values will rotate the glow anti-clockwise."],
					min = -2, max = 2, step = 0.01,
				},
				Length = {
					order = 3,
					type = "range",
					name = L["Length"],
					desc = L["Defines the length of each individual glow lines."],
					min = 2, max = 16, step = 1,
				},
				Thickness = {
					order = 4,
					type = "range",
					name = L["Thickness"],
					desc = L["Defines the thickness of the glow lines."],
					min = 1, max = 6, step = 1,
				},
				XOffset = {
					order = 5,
					type = "range",
					name = L["X-Offset"],
					min = -5, max = 5, step = 1,
				},
				YOffset = {
					order = 6,
					type = "range",
					name = L["Y-Offset"],
					min = -5, max = 5, step = 1,
				},
			},
		},
		AutoCast = {
			type = "group",
			name = L["AutoCast Glow"],
			order = 3,
			get = function(info) return CG.db.char.AutoCast[ info[#info] ] end,
			set = function(info, value) CG.db.char.AutoCast[ info[#info] ] = value; CG:ActionbarGlow() end,
			args = {
				NumParticles = {
					order = 1,
					type = "range",
					name = L["Num Particles"],
					desc = L["Defines the number of particle groups. Each group contains 4 particles."],
					min = 4, max = 16, step = 1,
				},
				Frequency = {
					order = 2,
					type = "range",
					name = L["Frequency"],
					desc = L["Sets the animation speed of the glow. Negative values will rotate the glow anti-clockwise."],
					min = -2, max = 2, step = 0.01,
				},
				Scale = {
					order = 3,
					type = "range",
					name = L["Scale"],
					min = 0.5, max = 4, step = 0.1,
				},
				XOffset = {
					order = 4,
					type = "range",
					name = L["X-Offset"],
					min = -5, max = 5, step = 1,
				},
				YOffset = {
					order = 5,
					type = "range",
					name = L["Y-Offset"],
					min = -5, max = 5, step = 1,
				},
			},
		},
	},
}

function CG:OnInitialize()
	CG.db = LibStub("AceDB-3.0"):New("CoolGlowDB", defaults)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Cool Glow", options)
	CG.optionsDialog = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Cool Glow")
end

function CG:OnEnable()
    self:ActionbarGlow()
	
	self:RegisterChatCommand("cg", "ToggleConfig")
    self:RegisterChatCommand("coolglow", "ToggleConfig")
end

function CG:ToggleConfig()
	InterfaceOptionsFrame:Show()
	InterfaceOptionsFrame_OpenToCategory("Cool Glow")
end

local function StartGlow(button)
	local Color = {CG.db.char.Color.r, CG.db.char.Color.g, CG.db.char.Color.b, CG.db.char.Color.a or 1}
	if CG.db.char.style == "Button" then
		LCG.ButtonGlow_Start(button, Color, CG.db.char.Button.Frequency)
	elseif CG.db.char.style == "Pixel" then
		LCG.PixelGlow_Start(button, Color, CG.db.char.Pixel.NumLines, CG.db.char.Pixel.Frequency, CG.db.char.Pixel.Length, CG.db.char.Pixel.Thickness, CG.db.char.Pixel.XOffset, CG.db.char.Pixel.YOffset)
	elseif CG.db.char.style == "AutoCast" then
		LCG.AutoCastGlow_Start(button, Color, CG.db.char.AutoCast.NumParticles, CG.db.char.AutoCast.Frequency, CG.db.char.AutoCast.Scale, CG.db.char.AutoCast.XOffset, CG.db.char.AutoCast.YOffset)
	end
end

local function StopGlow(button)
	LCG.ButtonGlow_Stop(button)
	LCG.PixelGlow_Stop(button)
	LCG.AutoCastGlow_Stop(button)
end

function CG:ActionbarGlow()
	if IsAddOnLoaded("ElvUI") or IsAddOnLoaded("Bartender4") then
		C_Timer.After(0,function()
			if LibStub then
				local lib = LibStub:GetLibrary("LibButtonGlow-1.0",4)
				if lib then
					function lib.ShowOverlayGlow(button)
						if button:GetAttribute("type") == "action" then
							local actionType,actionID = GetActionInfo(button:GetAttribute("action"))
							StartGlow(button)
						end
						function lib.HideOverlayGlow(button)
							StopGlow(button)
						end
					end
				end
			end
		end)
	else
		hooksecurefunc("ActionButton_ShowOverlayGlow", function(self)
			ActionButton_OverlayGlowAnimOutFinished(self.overlay.animOut)
			StartGlow(self)
		end)
		
		hooksecurefunc("ActionButton_HideOverlayGlow", function(self)
			StopGlow(self)
		end)
	end
end